import { PRE_GAME, PRE_ROUND, IN_ROUND, POST_ROUND, POST_GAME } from "./GameState";
import { createStore } from "../store";
import { createGame, joinGame, sendGuess, getCurrentRound } from "./GGSHService";

export const [
  {
    useGameId,
    usePlayerName,
    useLastRound,
    useGameJoined,
    useGameState,
  },
  dispatch,
] = createStore({
  gameId: null,
  playerName: null,
  lastRound: {
    roundNum: -1,
    selectedPoint: {
      lat: 0,
      lng: 0,
    },
    targetPoint: {
      lat: 0,
      lng: 0,
    },
    score: -1,
    totalScore: -1,
  },
  gameJoined: false,
  gameState: PRE_GAME,
}, {
  setGameId: ([set], gameId) => set({ gameId }),
  getGameId: ([_, get]) => get.gameId(),
  setPlayerName: ([set], playerName) => set({ playerName }),
  getPlayerName: ([_, get]) => get.playerName(),
  resetGame: ([set]) => set({ gameJoined: false, gameState: PRE_GAME }),
  startGame: ([set]) => set({ gameState: PRE_ROUND }),
  startRound: ([set]) => set({ gameState: IN_ROUND }),
  endGame: ([set]) => set({ gameState: POST_GAME }),
  createGameAndStart: async ([set, get], timer) => {
    const name = get.playerName();
    const gameId = await createGame(name, timer);
    set({ gameId, gameJoined: true, gameState: PRE_ROUND });
  },
  joinGame: async ([set, get]) => {
    const gameId = get.gameId();
    const name = get.playerName();
    await joinGame(gameId, name);
    set({ gameJoined: true });
  },
  submitGuess: async ([set, get], selectedPoint) => {
    const gameId = get.gameId();
    const name = get.playerName();
    const { currentRound, coord } = await getCurrentRound(gameId, name);
    const { score, totalScore } = await sendGuess(
      gameId,
      name,
      currentRound, 
      selectedPoint || { timeout: true }
    );
    set({
      lastRound: {
        roundNum: currentRound,
        selectedPoint,
        targetPoint: coord,
        score,
        totalScore,
      },
      gameState: POST_ROUND,
    });
  }
});